/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             OID Definition
*  SCOPE :              OID Definition
*  FILE :               oid.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __oid_HXX
#define __oid_HXX

/* IMPORTS *******************************************************************/

#ifndef __types_HXX
#define __types_HXX
#include <sys/types.h>
#endif

#ifndef __sockrpc_HXX
#include "sockrpc.hxx"
#endif

#ifndef __define_H
#include "define.h"
#endif

#ifndef __class_HXX
#include "class.hxx"
#endif

#ifndef __yoofunc_HXX
#include "yoofunc.hxx"
#endif

#ifndef __var_H
#include "var.h"
#endif

#ifndef __yooexcept_HXX
#include "yooexcept.hxx"
#endif

/*****************************************************************************/

struct PidInd {
  unsigned int dummy1 : 24 ;
  unsigned int lev5 : 2 ;
  unsigned int lev4 : 3 ;
  unsigned int lev3 : 3 ;
  unsigned int lev2 : 3 ;
  unsigned int lev1 : 3 ;
  unsigned int lev0 : 16 ;
  unsigned int dummy0 : 10 ;
} ;

struct OidInd {
  unsigned int dummy1 : 24 ;
  unsigned int lev11 : 2 ;
  unsigned int lev10 : 3 ;
  unsigned int lev9 : 3 ;
  unsigned int lev8 : 3 ;
  unsigned int lev7 : 3 ;
  unsigned int lev6 : 3 ;
  unsigned int lev5 : 3 ;
  unsigned int lev4 : 3 ;
  unsigned int lev3 : 3 ;
  unsigned int lev2 : 3 ;
  unsigned int lev1 : 3 ;
  unsigned int lev0 : 8 ;
} ;


typedef enum { YOO_DummyValue } YOO_DummyType ;

struct PID {

  unsigned int sz      : 24 ; 	// Size in page for a page.
                                // Size in slot for an object
  unsigned int volNo   :  8 ; 	// Volume number 
  unsigned int pageNo  : 22 ;	// Page number
  unsigned int objNo   : 10 ;   // object number
  
  PID(u_int vol=0, u_int page=0, u_int obj=0) ;
  PID(YOO_DummyType) {} ;
  PID& operator+=(int) ;
  int operator==(const PID& pid) const ;
  int operator!=(const PID& pid) const ;
  u_int getRootInd() const ;
  u_int getNodeInd(int lev) const ;
  void setsize(int s) ;
  u_int size() const ;
} ;


struct OID : public PID {
  OID(u_int vol=0, u_int page=0, u_int obj=0) ;
  OID(PID& pid) ;
  OID(YOO_DummyType) : PID(YOO_DummyValue) {} ;
  PID pid() const ;
  int operator==(const OID& oid) const ;
  int operator!=(const OID& oid) const ;
  int operator<(const OID& oid) const ;
  int operator>(const OID& oid) const ;
  u_int getRootInd() const ;
  u_int getNodeInd(int lev) const ;
} ;


const OID NULL_OID(0, 0, 0) ;
const PID NULL_PID(0, 0, 0) ;

inline PID::PID(u_int v, u_int p, u_int o)
{
  if (YOO_LOAD_TIME == FALSE) {
    volNo = v; pageNo = p; objNo = o; sz = 1;
  }
} 

inline int PID::operator==(const PID& pid) const {
  return ((this->volNo==pid.volNo)&&(this->pageNo==pid.pageNo)) ;
}

inline int PID::operator!=(const PID& pid) const {
  return !(*this == pid) ;
}

inline void PID::setsize(int s) {
  this->sz = s ;
}

inline u_int PID::size() const {
  return sz ;
}

inline PID& PID::operator+=(int p) {
  this->pageNo += p ;  
  return *this ;
}

inline  u_int PID::getRootInd() const {
  return ((PidInd*)this)->lev0 ;
}

inline u_int PID::getNodeInd(int lev) const {
  if (lev == 1) return ((PidInd*)this)->lev1 ;
  else if (lev == 2) return ((PidInd*)this)->lev2 ;
  else if (lev == 3) return ((PidInd*)this)->lev3 ;
  else if (lev == 4) return ((PidInd*)this)->lev4 ;
  else if (lev == 5) return ((PidInd*)this)->lev5 ;
  else if (lev == 0) return ((PidInd*)this)->lev0 ;
  else yoo_exit(YOO_ERR_DBG) ;
  return 0 ; /* never reached */
}    

inline OID::OID(u_int v, u_int p, u_int o) : PID(v, p, o) {}

inline OID::OID(PID& pid) : PID(pid) {}

inline int OID::operator==(const OID& oid) const {
  return ((this->volNo==oid.volNo)&&(this->pageNo==oid.pageNo)&&
	  (this->objNo==oid.objNo)) ;
}

inline int OID::operator!=(const OID& oid) const {
  return !(*this == oid) ;
}

inline int OID::operator<(const OID& oid) const {
  return ( (volNo<oid.volNo) ||
	   ((volNo==oid.volNo)&&(pageNo<oid.pageNo)) ||
	   ((volNo==oid.volNo)&&(pageNo==oid.pageNo)&&(objNo<oid.objNo)) ) ;
}

inline int OID::operator>(const OID& oid) const {
  return !(*this < oid) ;
}

inline  PID OID::pid() const {
  return PID(volNo, pageNo) ;
}

inline  u_int OID::getRootInd() const {
  return ((OidInd*)this)->lev0 ;
}

inline u_int OID::getNodeInd(int lev) const {
  if (lev == 1) return ((OidInd*)this)->lev1 ;
  else if (lev == 2) return ((OidInd*)this)->lev2 ;
  else if (lev == 3) return ((OidInd*)this)->lev3 ;
  else if (lev == 4) return ((OidInd*)this)->lev4 ;
  else if (lev == 5) return ((OidInd*)this)->lev5 ;
  else if (lev == 6) return ((OidInd*)this)->lev6 ;
  else if (lev == 6) return ((OidInd*)this)->lev7 ;
  else if (lev == 6) return ((OidInd*)this)->lev8 ;
  else if (lev == 6) return ((OidInd*)this)->lev9 ;
  else if (lev == 6) return ((OidInd*)this)->lev10 ;
  else if (lev == 6) return ((OidInd*)this)->lev11 ;
  else if (lev == 0) return ((OidInd*)this)->lev0 ;
  else yoo_exit(YOO_ERR_DBG) ;
  return 0 ; /* never reached */
}    

inline SvcSocket& operator<<(SvcSocket& s, const OID& oid) { 
  s.send((char*)&oid, sizeof(OID)) ; return s ;
}
inline SvcSocket& operator>>(SvcSocket& s, OID& oid) { 
  s.recv((char*)&oid, sizeof(OID)) ; return s ;
}

inline SvcSocket& operator<<(SvcSocket& s, const PID& pid) { 
  s.send((char*)&pid, sizeof(PID)) ; return s ;
}
inline SvcSocket& operator>>(SvcSocket& s, PID& pid) { 
  s.recv((char*)&pid, sizeof(PID)) ; return s ;
}




#endif
